import numpy as np
from pyxdameraulevenshtein import damerau_levenshtein_distance_ndarray


def which_item(recall, current_list, prior_lists, dictionary):

    """
    :param recall:          string              the word typed in by the subject at recall
    :param current_list:    list of strings     words seen by a subject during the current trial
    :param prior_lists:     list of strings     all words seen by a subject up until the current trial
    :param dictionary:      list of strings     all words to be considered as intrusions (typically the full English dictionary -- websters_dict.txt)
    :return:
    """

    # strip any whitespace from the recalled string
    recall = recall.replace(" ", "")

    # Make sure the dictionary and the recall are both all in upper case.
    # Doing this will be expensive and inefficient because this gets run for
    # EVERY word recalled in an experiment, but it is worth it to avoid errors
    if sum([word.islower() for word in dictionary]) == 0:
        recall = recall.upper()
    else:
        raise ValueError('Error with Dictionary')

    # figure out what was recalled
    all_lists = current_list + prior_lists
    if not recall.isalpha(): # check for uncorrectable typing errors (space, number, empty string or any non-alpha)
        return -4, recall
    if recall in all_lists: # see if the recall exactly matches a word that has been presented
        seen_where = all_lists.index(recall)
        if seen_where > len(current_list)-1:  # is it an intrusion from a prior list (PLI)?
            return -2, recall
        else:  # its not a PLI, so return its serial position (just seen_where)
            return seen_where, recall
    elif recall in dictionary:  # see if the recall exactly match a word in the dictionary
        return -3, recall
    else:  # correct spelling and start over
        recall = correct_spelling(recall, all_lists, dictionary)
        return which_item(recall, current_list, prior_lists, dictionary)


def correct_spelling(recall, current_list, dictionary):

    """
    :param recall:          string              the word typed in by the subject at recall
    :param current_list:    list of strings     words seen by a subject during the current trial
    :param dictionary:      list of strings     all words to be considered as intrusions (typically the full English dictionary -- websters_dict.txt)
    :return:
    """

    # edit distance to each item in the pool and dictionary
    dist_to_pool = damerau_levenshtein_distance_ndarray(recall, np.array(current_list))
    dist_to_dict = damerau_levenshtein_distance_ndarray(recall, np.array(dictionary))

    # position in distribution of dist_to_dict
    ptile = np.true_divide(sum(dist_to_dict <= np.amin(dist_to_pool)), dist_to_dict.size)

    # decide if it is a word in the pool or an ELI
    if ptile <= .1:
        corrected_recall = current_list[np.argmin(dist_to_pool)]
    else:
        corrected_recall = dictionary[np.argmin(dist_to_dict)]
    return corrected_recall